import contextlib

from jaraco.path import tempfile_context


def sheet_as_dict(worksheet):
    """
    Take an xsrd worksheet and convert it to a dict, using the first
    row as a header (to create keys for the subsequent rows).
    """
    keys = worksheet.row_values(0)
    value_range = range(1, worksheet.nrows)

    def to_dict(values):
        return dict(zip(keys, values))

    return [to_dict(worksheet.row_values(n)) for n in value_range]


# constants from http://msdn.microsoft.com/en-us/library/bb241279.aspx
xlXMLSpreadsheet = 46


def open_workbook(filename):
    from win32com.client import Dispatch

    app = Dispatch('Excel.Application')
    return app.Workbooks.Open(filename)


@contextlib.contextmanager
def suppress_alerts(workbook):
    app = workbook.application
    previous_state = app.DisplayAlerts
    app.DisplayAlerts = False
    try:
        yield
    finally:
        app.DisplayAlerts = previous_state


def get_workbook_as_xml(path):
    """
    Use Office 2003 or Office 2007 to open up a .xls workbook, save
    it as the Office 2003 Excel XML Spreadsheet format in a temporary
    file, read that XML into memory, then delete the temporary file.
    """
    workbook = open_workbook(path)
    with tempfile_context(suffix='.xml') as filename:
        with suppress_alerts(workbook):
            workbook.SaveAs(Filename=filename, FileFormat=xlXMLSpreadsheet)
            workbook.Close(SaveChanges=0)
        with open(filename, 'rb') as datafile:
            data = datafile.read()
    return data



import inspect
import logging
import itertools
from glob import glob
from optparse import OptionParser

from jaraco.path import ExtensionReplacer

from .convert import Converter


log = logging.getLogger(__name__)


def save_to(content, filename):
    open(filename, 'wb').write(content)


def handle_multiple(docfile, pdffile=None):
    """
    Handle docfile if it matches more than one file.
    """
    doc_files = glob(docfile)
    n_files = len(doc_files)
    if n_files > 1:
        if pdffile is not None:
            raise Exception("Cannot specify output file with multiple sources")
    log.info("Processing {n_files} source files...".format(**vars()))
    converter = Converter()
    doc_content = itertools.imap(lambda f: open(f, 'rb').read(), doc_files)
    pdf_content = itertools.imap(converter, doc_content)
    pdf_files = (
        itertools.imap(ExtensionReplacer('.pdf'), doc_files)
        if not pdffile
        else [pdffile]
    )
    map(save_to, pdf_content, pdf_files)


def doc_to_pdf_cmd():
    """
    %prog <word doc> [<pdf file>]

    Convert a word document to a PDF using Office 2007
    """
    usage = inspect.getdoc(doc_to_pdf_cmd)
    parser = OptionParser(usage=usage)
    options, args = parser.parse_args()
    if not 1 <= len(args) <= 2:
        parser.error("Incorrect number of arguments")
    logging.basicConfig(level=logging.INFO)
    handle_multiple(*args)



"""
Build script to create a doc-to-pdf convert server as a Windows executable.
"""

import os
import textwrap


setup_params = dict(
    console=['server.py'],
    options=dict(
        py2exe=dict(
            packages=['pkg_resources'],
        ),
    ),
    script_args=('py2exe',),
)

if __name__ == '__main__':
    from setuptools import setup

    __import__('py2exe')
    code = """
        from jaraco.office import convert
        convert.ConvertServer.start_server()
        """
    open('server.py', 'w').write(textwrap.dedent(code))
    setup(**setup_params)
    os.remove('server.py')



import os
import argparse
from contextlib import contextmanager

from jaraco.path import save_to_file, replace_extension


@contextmanager
def word_context(word, filename, close_flags):
    doc = word.Documents.Open(filename)
    try:
        yield doc
    finally:
        doc.Close(close_flags)


class Converter:
    """
    An object that will convert a Word-readable file to one of the Word-
    savable formats (defaults to PDF).

    Requires Microsoft Word 2007 or later.
    """

    def __init__(self):
        from win32com.client import Dispatch
        import pythoncom
        import threading

        if threading.current_thread().getName() != 'MainThread':
            pythoncom.CoInitialize()
        self.word = Dispatch('Word.Application')

    def convert(self, docfile_string, target_format=None):
        """
        Take a string (in memory) and return it as a string of the
        target format (also as a string in memory).
        """
        from win32com.client import constants

        target_format = target_format or getattr(constants, 'wdFormatPDF', 17)

        with save_to_file(docfile_string) as docfile:
            # if I don't put a pdf extension on it, Word will
            pdffile = replace_extension('.pdf', docfile)
            dont_save = getattr(constants, 'wdDoNotSaveChanges', 0)
            with word_context(self.word, docfile, dont_save) as doc:
                doc.SaveAs(pdffile, target_format)
            with open(pdffile, 'rb') as pdf:
                content = pdf.read()
            os.remove(pdffile)
        return content

    __call__ = convert

    def __del__(self):
        self.word.Quit()


form = """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" \
"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
    xmlns:py="http://genshi.edgewall.org/"
    xmlns:xi="http://www.w3.org/2001/XInclude">
<head>
</head>
<body>
    <div>
        <form method="post" action="convert" enctype="multipart/form-data">
            <input type="file" name="document"/>
            <input type="submit" />
        </form>
    </div>
</body>
</html>""".replace(
    '    ', '\t'
)


class ConvertServer:
    def index(self):
        return form

    index.exposed = True  # type: ignore

    def convert(self, document):
        cherrypy.response.headers['Content-Type'] = 'application/pdf'
        return Converter().convert(document.file.read())

    convert.exposed = True  # type: ignore

    @staticmethod
    def start_server():
        global cherrypy
        import cherrypy

        parser = argparse.ArgumentParser()
        parser.add_argument('config')
        args = parser.parse_args()
        cherrypy.quickstart(ConvertServer(), config=args.config)



import re

from win32com.client import Dispatch


try:
    app = Dispatch('Excel.Application')
except Exception:
    app = None

# wb = app.Workbooks.Open(filename)

default_pattern = 'Total'


def CleanRow(row):
    return filter(None, row.Value[0])


def HasPattern(row, pattern=default_pattern):
    strings = map(str, CleanRow(row))
    pattern = re.compile(pattern, re.I)
    return bool(filter(pattern.search, strings))


def NotHasPattern(row, pattern=default_pattern):
    return not HasPattern(row, pattern)


def IsNotBlank(row):
    return bool(CleanRow(row))


def IsBlank(row):
    return not IsNotBlank(row)


def DeleteUnmatchedRows():
    """
    Delete rows that don't match the pattern; you'll have to run this
    multiple times if you want to get consecutive matches
    """
    # excel doesn't support iteration
    # valueRows = takewhile(IsNotBlank, app.Rows)
    # map(DeleteRow, filter(NotHasPattern, valueRows))
    for row in GetPopulatedRows():
        if NotHasPattern(row):
            print('deleting row', CleanRow(row))
            row.Delete()


def GetPopulatedRows():
    for row in app.Rows:
        if IsBlank(row):
            break
        yield row


def FixBlankWPNCD(row):
    if row.Cells(1, 1).Value is None:
        app.Cells(row.Row, row.Column).Value = app.Cells(row.Row - 1, row.Column).Value
